<?php

/**
 *
 * $Id: class.TaskAdminCredentialsChecker.php,v 1.2 2005/04/29 22:37:55 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.2 $
 *
 */

/**
 * coWiki - Installer task class
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
class TaskAdminCredentialsChecker extends AbstractTask {

    public function __construct($Tpl) {
        parent::__construct($Tpl);
    }

    // --------------------------------------------------------------------

    public function isRequired() {
        if ($this->State->isValidUser()) {
            return false;
        }
        return true;
    }

    // --------------------------------------------------------------------

    public function validate() {

        // Check request data
        if (!isset($_POST['passwd'])) {
            return false;
        }

        $sLogin = 'root';
        $sPasswd = $_POST['passwd'];

        // Get the root password if this is an initial setup
        if ($this->State->isInitialSetup()) {

            if (strlen($sPasswd) < 5) {
                return $this->addError(
                    'The password is too short and hence unsecure.
                    Enter a longer (better) password.'
                );
            }

            $this->State->setRootUserPassword($sPasswd);
            $this->State->setIsValidUser(true);

            return true;
        }

        // Check the root password if this is an update
        if ($this->State->isUpdateSetup()) {

            // If we are updating, we require a valid ClassLoader of the
            // application we are setting up. If the ClassLoader is not
            // available, jump to the beginning. FIX: better solution,
            // redirection is confusing.
            if (!$this->hasValidClassLoader()) {
                redirectToController();
            }

            // ---

            // Load the main application ClassLoader
            include_once $this->State->getCompleteClassLoaderPath();

            // Get configuration
            $Conf = new Config($this->State->getIncludesPath().'/core.conf');

            // Get the 'root' user password
            $sRootPass = $Conf->getSectionKeyValue('.AUTH', 'ROOT_PASSWD');

            // Check password
            if ($sLogin == 'root') {

                if (crypt($sPasswd, $sRootPass) == $sRootPass) {

                    // Must be set as it will crypted at the end.
                    $this->State->setRootUserPassword($sPasswd);

                    // Set the user valid
                    $this->State->setIsValidUser(true);
                    return true;
                }
            }
        }

        return false;
    }

    // --------------------------------------------------------------------

    public function render() {

        // Reset user, if user comes along here
        $this->State->setIsValidUser(false);

        // ----------------------------------------------------------------

        if ($this->State->isInitialSetup()) {
            $this->addInfo(
                'The coWiki setup wizard has decteted that this is its frist
                 invocation.
                 Please enter your administrator (<tt>root</tt> user)
                 password for coWiki that you are going to use in the future.
                 Please do not forget this password.
                 '
            );
        }

        $sStr = '<br/><br/>';

        // Visual space if none of the text about is printed
        if (!$this->hasError()) {
            $sStr = '<br/><br/>';
        }

        // ----------------------------------------------------------------

        $sStr .=  '<table align="center" cellspacing="0" cellpadding="0"
                     border="0">';
        $sStr .=    '<tr>';

        $sStr .=      '<td>';
        $sStr .=        '<fieldset>';
        $sStr .=          '<legend>coWiki administrator (root) login</legend>';

        $sStr .=          '<table cellpadding="2" cellspacing="0"';
        $sStr .=          ' border="0" class="fieldset-content">';

        $sStr .=            '<tr>';
        $sStr .=              '<td align="right">';
        $sStr .=                'Login:';
        $sStr .=              '</td>';
        $sStr .=              '<td>';
        $sStr .=                '<input type="text" name="login"';
        $sStr .=                ' value="root" disabled="true">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td align="right">';
        $sStr .=                'Password:';
        $sStr .=              '</td>';
        $sStr .=              '<td>';
        $sStr .=                '<input type="password" name="passwd">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=          '</table>';
        $sStr .=        '</fieldset>';
        $sStr .=      '</td>';

        $sStr .=    '</tr>';
        $sStr .=  '</table>';

        $sStr .=  '<br/>';

        // Define content
        $this->Tpl->assign('CONTENT', $sStr);

        // Define informational/warning/error messages
        $this->Tpl->assign('MESSAGE', $this->getMsgQueueFormatted());

        echo $this->Tpl->parse(SETUP_PATH.'/tpl/window.default.tpl');
    }

} // of class

/*
    Go son, go down to the water
    And see the women weeping there
    Then go up into the mountains
    The men, they are weeping too

    Father, why are all the women weeping?
    They are weeping for their men
    Then why are all the men there weeping?
    They are weeping back at them

    This is a weeping song
    A song in which to weep
    While all the men and women sleep
    This is a weeping song
    But I won't be weeping long

    Father, why are all the children weeping?
    They are merely crying son
    O, are they merely crying, father?
    Yes, true weeping is yet to come

    This is a weeping song
    A song in which to weep
    While all the men and women sleep
    This is a weeping song
    But I won't be weeping long

    O father tell me, are you weeping?
    Your face seems wet to touch
    O then I'm so sorry, father
    I never thought I hurt you so much

    This is a weeping song
    A song in which to weep
    While we rock ourselves to sleep
    This is a weeping song
    But I won't be weeping long
*/

?>