<?php

/**
 *
 * $Id: class.TaskApplicationVisualSettings.php,v 1.2 2005/04/26 05:24:58 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.2 $
 *
 */

/**
 * coWiki - Installer task class
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
class TaskApplicationVisualSettings extends AbstractTask {

    // --------------------------------------------------------------------

    public function __construct($Tpl) {
        parent::__construct($Tpl);
        parent::checkValidUser();
        parent::loadClassLoader();

        // ---

        // Load configuration data only once. Keep it in sesssion (state
        // holder) then
        if (!$this->State->hasVisualSettings()) {

            // Load application configuration
            $Conf = $this->getConfig();

            // Initialize state holder values
            $this->State->setLanguageLocale(
                $Conf->getSectionKeyValue('RUNTIME', 'LANGUAGE_LOCALE')
            );
            $this->State->setTemplateActive(
                $Conf->getSectionKeyValue('RUNTIME', 'TEMPLATE_ACTIVE')
            );

            // Get email obfuscation state
            $this->State->setHasEmailObfuscation(
                $Conf->getSectionKeyValueBoolean(
                    'RUNTIME', 'EMAIL_OBFUSCATE'
                )
            );

            // Do we need to display the processing time?
            $this->State->setShowProcessingTime(
                $Conf->getSectionKeyValueBoolean(
                    'RUNTIME', 'SHOW_PROCESSING_TIME'
                )
            );


            $this->State->setHasVisualSettings(true);
        }
    }

    // --------------------------------------------------------------------

    public function isRequired() {
        return true;
    }

    // --------------------------------------------------------------------

    public function validate() {

        // Store submitted values
        $this->State->setTemplateActive(@$_POST['template']);
        $this->State->setLanguageLocale(@$_POST['locale']);

        $this->State->setHasEmailObfuscation(isset($_POST['obfuscate']));
        $this->State->setShowProcessingTime(isset($_POST['processing']));

        return true;
    }

    // --------------------------------------------------------------------

    public function render() {

        // Get available themes
        $aTempl = array();
        $sDir = $this->State->getHtdocsPath().'/tpl/';

        // Read all template directories
        $rDir = @opendir($sDir);

        while ($sDirName = @readdir($rDir)) {

            // Skip unwanted files
            if ($sDirName{0} == '.') { continue; }
            if (strtolower($sDirName) == 'cvs') { continue; }

            // Read template information file
            $sFileName = '/tpl/' . $sDirName . '/tpl.info';
            $Conf = new Config($this->State->getHtdocsPath() . $sFileName);

            $sTplName = $Conf->getSectionKeyValue('TEMPLATE', 'NAME');
            $sTplDesc = $Conf->getSectionKeyValue('TEMPLATE', 'DESCRIPTION');
            
            // Any description given?
            if ($sTplDesc != '') { $sTplName .= ' ('.$sTplDesc.')'; }

            $aTempl[$sDirName] = $sTplName;
        }

        // ----------------------------------------------------------------

        // Get available translations (languages)
        $aLocale = array();
        $sDir = $this->State->getIncludesPath().'/locale/';

        // Read all template directories
        $rDir = @opendir($sDir);

        while ($sFileName = @readdir($rDir)) {

            // Skip unwanted files
            if ($sFileName{0} == '.') { continue; }
            if (is_dir($sDir . $sFileName)) { continue; }
            if ($sFileName == 'error.cat') { continue; }

            // Skip locale file that contain 'mail' in their name
            if (preg_match('#mail#i', $sFileName)) {
                continue;
            }

            // Avoid errors in the loop, if I18N is not set. Reset.
            $__ = array();
            $sName = substr($sFileName, 0, -4);

            // Assume that the language string is not set in the I18N file
            $sI18N = $sName;

            // Load I18N catalog files and fetch the language string of
            // each of them.
            if (@include($sDir . $sFileName)) {
                if (isset($__['I18N_CATALOG_LANGUAGE_ENGLISH_STRING'])) {
                    $sLang = $__['I18N_CATALOG_LANGUAGE_ENGLISH_STRING'];
                    $sI18N = $sLang . ' ('.$sName.')';
                }
            }

            $aLocale[] = array('string' => $sI18N, 'file' => $sName);
        }

        sort($aLocale);

        // ----------------------------------------------------------------

        $sStr = 'Determine the primary coWiki appearance and the
                default language settings. Depending on the template and
                functionality you choose, a user will be still able to
                change these setting in his personal preferences menu.
                <br/><br/>

                What you set here, are the default visual settings for
                your guest users who have not yet chosen any other theme
                or appearance for their own.';


        $sStr .=  '<br/><br/>';

        $sStr .=  '<table align="center" cellspacing="0"
                    cellpadding="0" border="0">';
        $sStr .=    '<tr>';

        $sStr .=      '<td>';
        $sStr .=        '<fieldset>';
        $sStr .=          '<legend>coWiki primary appearance</legend>';

        $sStr .=          '<table cellpadding="2" cellspacing="0"';
        $sStr .=          ' border="0" class="fieldset-content">';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Template:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="3">';
        $sStr .=                '<select name="template" style="width:100%">';

        foreach ($aTempl as $k => $v) {
            $sStr .=              '<option value="'.$k.'"';

            if ($this->State->getTemplateActive() == $k) {
                $sStr .=  ' selected="true"';
            }

            $sStr .=              '>' . $v . '</option>';
        }

        $sStr .=                '</select>';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Language:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="3">';

        $sStr .=                '<select name="locale" style="width:100%">';

        foreach ($aLocale as $v) {
            $sStr .=              '<option value="'.$v['file'].'"';

            if ($this->State->getLanguageLocale() == $v['file']) {
                $sStr .=  ' selected="true"';
            }

            $sStr .=              '>' . $v['string'] . '</option>';
        }

        $sStr .=                '</select>';

        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td colspan="3">';

        // Horizontal separator
        $sStr .=                SETUP_FIELDSET_SEPARATOR;

        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td>&nbsp;</td>';

        $sStr .=              '<td>';
        $sStr .=                '<input name="obfuscate" type="checkbox"';

        if ($this->State->hasEmailObfuscation()) { $sStr .= ' checked="true"'; }

        $sStr .=                ' id="obfuscate">';
        $sStr .=              '</td>';
        $sStr .=              '<td nowrap>';
        $sStr .=                '<label for="obfuscate">';
        $sStr .=                  'Obfuscate emails (avoid spam on emails)';
        $sStr .=                '</label>';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td>&nbsp;</td>';

        $sStr .=              '<td>';
        $sStr .=                '<input name="processing" type="checkbox"';

        if ($this->State->showProcessingTime()) { $sStr .= ' checked="true"'; }

        $sStr .=                ' id="processing">';
        $sStr .=              '</td>';
        $sStr .=              '<td nowrap>';
        $sStr .=                '<label for="processing">';
        $sStr .=                  'Show request processing time (template
                                   dependant)';
        $sStr .=                '</label>';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=          '</table>';
        $sStr .=        '</fieldset>';
        $sStr .=      '</td>';

        $sStr .=    '</tr>';
        $sStr .=  '</table>';

        $sStr .=  '<br/>';

        // Define content
        $this->Tpl->assign('CONTENT', $sStr);

        // Define informational/warning/error messages
        $this->Tpl->assign('MESSAGE', $this->getMsgQueueFormatted());

        echo $this->Tpl->parse(SETUP_PATH.'/tpl/window.default.tpl');
    }

} // of class

?>