<?php

/**
 *
 * $Id: class.TaskDocumentStorageAdminUser.php,v 1.3 2005/05/12 19:40:56 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.3 $
 *
 */

/**
 * coWiki - Installer task class
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
class TaskDocumentStorageAdminUser extends AbstractTask {

    private
        $Conf = null;

    // --------------------------------------------------------------------

    public function __construct($Tpl) {
        parent::__construct($Tpl);
        parent::checkValidUser();
        parent::loadClassLoader();
    }

    // --------------------------------------------------------------------

    public function isRequired() {
        return $this->State->shallCreateNewDocumentDatabase();
    }

    // --------------------------------------------------------------------

    public function validate() {

        // Check submitted values
        if (@$_POST['user'] == '') {
            $this->addError(
                'You did not enter the credentials of the privileged user
                who has administrative access to your database.'
            );
        }

        // Store submitted values
        $this->State->setDocStorageAdminUser(@$_POST['user']);
        $this->State->setDocStorageAdminPassword(@$_POST['passwd']);

        return !$this->hasError();
    }

    // --------------------------------------------------------------------

    public function render() {

        $sStr =   'You have decided to create a new database and/or a
                  new connector user for coWiki.
                  To achieve this, the installer needs the credentials
                  of the user who is able to create such a new database
                  or such a user.
                  Usually this is the <tt>root</tt> user (Administrator,
                  DBA) of the database, but also might be an other user
                  with administrative access who has been granted the
                  appropriate privileges for this task.
                  <br/><br/>

                  The credentials you enter here, won\'t be stored anywhere
                  (except in the running session). They are only required
                  temporarily during this setup.
                  <br/><br/>

                  <span class="warn">
                    <b>Attention:</b> An administrator can destroy the
                    <i>complete</i> database! Be careful.
                  </span>
                  ';
                  
        $sStr .=  '<br/><br/>';

        $sStr .=  '<table align="center" cellspacing="0"
                    cellpadding="0" border="0">';
        $sStr .=    '<tr>';

        $sStr .=      '<td>';
        $sStr .=        '<fieldset>';
        $sStr .=          '<legend>Document database privileged user</legend>';

        $sStr .=          '<table cellpadding="2" cellspacing="0"';
        $sStr .=          ' border="0" class="fieldset-content">';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Database administrator:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="3">';
        $sStr .=                '<input type="text" name="user"';
        $sStr .=                ' size="20" value="';
        $sStr .=                  $this->State->getDocStorageAdminUser();
        $sStr .=                '">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Password:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="3">';
        $sStr .=                '<input type="password" name="passwd"';
        $sStr .=                ' size="20" value="';
        $sStr .=                  $this->State->getDocStorageAdminPassword();
        $sStr .=                '">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=          '</table>';
        $sStr .=        '</fieldset>';
        $sStr .=      '</td>';

        $sStr .=    '</tr>';
        $sStr .=  '</table>';

        $sStr .=  '<br/>';

        // Define content
        $this->Tpl->assign('CONTENT', $sStr);

        // Define informational/warning/error messages
        $this->Tpl->assign('MESSAGE', $this->getMsgQueueFormatted());

        echo $this->Tpl->parse(SETUP_PATH.'/tpl/window.default.tpl');
    }

} // of class

?>