<?php

/**
 *
 * $Id: class.TaskDocumentStorageConnector.php,v 1.12 2006/01/24 01:01:55 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.12 $
 *
 */

/**
 * coWiki - Installer task class
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
class TaskDocumentStorageConnector extends AbstractTask {

    const MYSQL_REQUIRED_VERSION        = '4.0.0';
    const MYSQLI_REQUIRED_VERSION       = '4.1.0';
    const POSTGRESQL_REQUIRED_VERSION   = '7.4.5';
    const SQLITE_REQUIRED_VERSION       = '3.2.0';

    private
        $Conf = null;

    // --------------------------------------------------------------------

    public function __construct($Tpl) {
        parent::__construct($Tpl);
        parent::checkValidUser();
        parent::loadClassLoader();

        // ---

        // Load configuration data only once. Keep it in sesssion (state
        // holder) then
        if (!$this->State->hasDocStorageResources()) {

            // Load application configuration
            $Conf = $this->getConfig();

            // Initialize state holder values
            $UriInfo = new UriInfo(
                          $Conf->getSectionKeyValue(
                            '.DOCUMENT', 'RESOURCE'
                          )
                       );

            $this->State->setDocStorageConnectorUser($UriInfo->get('user'));
            $this->State->setDocStorageConnectorPassword($UriInfo->get('pass'));
            $this->State->setDocHost($UriInfo->get('host'));
            $this->State->setDocHostPort($UriInfo->get('port'));
            $this->State->setDocDatabaseName($UriInfo->get('basepath'));

            $this->State->setHasDocStorageResources(true);
        }

        // Make sure, that everytime we come across this task, the
        // following flags are resetted.
        $this->State->setHasValidDocStorageConnection(false);
    }

    // --------------------------------------------------------------------

    public function isRequired() {
        return true;
    }

    // --------------------------------------------------------------------

    public function validate() {

        // Clean, minus character ("-") is not allowed.
        $_POST['user'] = preg_replace(
                            '#[^a-z0-9_]#i',
                            '',
                            @$_POST['user']
                         );

        // Clean, minus character ("-") is not allowed.
        $_POST['database'] = preg_replace(
                                '#[^a-z0-9_]#i',
                                '',
                                @$_POST['database']
                             );

        // Store submitted values
        $this->State->setDocStorageConnectorUser(@$_POST['user']);
        $this->State->setDocStorageConnectorPassword(@$_POST['passwd']);
        $this->State->setDocHost(@$_POST['host']);
        $this->State->setDocHostPort(@$_POST['port']);
        $this->State->setDocDatabaseName(@$_POST['database']);

        // Complain about stupid settings (e.g. example settings taken
        // directly for the core.conf-dist file)
        if ($_POST['user'] == 'USER' || $_POST['user'] == '') {
            $this->addErr('Use an other name for the connector user.');
        }
        if ($_POST['passwd'] == 'PASSWD' || $_POST['passwd'] == '') {
            $this->addErr('Use an other password for the connector user.');
        }
        if ($_POST['host'] == '%' || $_POST['host'] == '') {
            $this->addErr('Use an other address for your host.');
        }
        if ($_POST['database'] == 'DATABASE' || $_POST['database'] == '') {
            $this->addErr('Use an other name for the database.');
        }

        if ($this->hasMessage()) {
            return !$this->hasMessage();
        }

        // ---

        // Try to connect the database and check _connector_ user privileges
        $this->checkDBConnection();

        return !$this->hasMessage();
    }

    // --------------------------------------------------------------------

    public function render() {

        $sStr =   'The main coWiki documents are stored in
                   the database you define here. Depending on your
                   installation setup, the authentification of users
                   and the source of user data (like names, emails and
                   other personal data) can be defined <i>separately</i>
                   in the next steps of this wizard.
                   <br/><br/>

                   These database settings define the connector
                   user that is allowed to connect, access and alter the
                   main coWiki database only for storing documents.
                   You may leave "Port" empty for the standard database
                   access port of the storage container you use.';

        $sStr .=  '<br/><br/>';

        $sStr .=  '<table align="center" cellspacing="0"
                    cellpadding="0" border="0">';
        $sStr .=    '<tr>';

        $sStr .=      '<td>';
        $sStr .=        '<fieldset>';
        $sStr .=          '<legend>Document database connector data</legend>';

        $sStr .=          '<table cellpadding="2" cellspacing="0"';
        $sStr .=          ' border="0" class="fieldset-content">';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Storage container:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="3">';

        $aArr = $this->getConnectorSchemes();
        $sStr .= $aArr[$this->State->getDocStorageScheme()];

        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td colspan="4">';

        // Horizontal separator
        $sStr .=                SETUP_FIELDSET_SEPARATOR;

        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Connector user:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="3">';
        $sStr .=                '<input type="text" name="user"';
        $sStr .=                ' style="width:100%"';
        $sStr .=                ' value="';
        $sStr .=                  $this->State->
                                      getDocStorageConnectorUser();
        $sStr .=                '">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Connector user password:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="3">';
        $sStr .=                '<input type="text" name="passwd"';
        $sStr .=                ' style="width:100%"';
        $sStr .=                ' value="';
        $sStr .=                  $this->State
                                      ->getDocStorageConnectorPassword();
        $sStr .=                '">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Database host:';
        $sStr .=              '</td>';
        $sStr .=              '<td>';
        $sStr .=                '<input type="text" name="host" size="15"';
        $sStr .=                ' value="';
        $sStr .=                  $this->State->getDocHost();
        $sStr .=                '">';
        $sStr .=              '</td>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Port:';
        $sStr .=              '</td>';
        $sStr .=              '<td>';
        $sStr .=                '<input type="text" name="port" size="5"';
        $sStr .=                ' value="';
        $sStr .=                  $this->State->getDocHostPort();
        $sStr .=                '">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Database name:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="3">';
        $sStr .=                '<input type="text" name="database"';
        $sStr .=                ' style="width:100%"';
        $sStr .=                ' value="';
        $sStr .=                  $this->State->getDocDatabaseName();
        $sStr .=                '">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=          '</table>';
        $sStr .=        '</fieldset>';
        $sStr .=      '</td>';

        $sStr .=    '</tr>';
        $sStr .=  '</table>';

        $sStr .=  '<br/>';


        // Define content
        $this->Tpl->assign('CONTENT', $sStr);

        // Define informational/warning/error messages
        $this->Tpl->assign('MESSAGE', $this->getMsgQueueFormatted());

        echo $this->Tpl->parse(SETUP_PATH.'/tpl/window.default.tpl');
    }

    // ====================================================================

    // Basic connection check
    private function checkDBConnection() {

        try {
            $sResource = $this->State->shallCreateNewDocumentDatabase()
                            ? $this->State->getDocumentStorageRootDSN()
                            : $this->State->getDocumentStorageDSN();

            $UriInfo = new UriInfo($sResource);

            $Storage = StorageFactory::getInstance()->create($UriInfo);

        // Do we have basically the capability to access this database?
        } catch (StorageApiException $sae) {
            return $this->addError(
                'Your PHP installation basically does not support
                access to the database container you have chosen: '
                .$sae->getExceptionMessage()
            );

        } catch (StorageConnectionFailureException $scfe) {
            return $this->addError($scfe->getExceptionMessage());
        }

        // ---

        // If we should use an already existing database, select it
        if (!$this->State->shallCreateNewDocumentDatabase()) {
            try {
                $Storage->useDatabase($this->State->getDocDatabaseName());

                // Apparently the database exists and is accessible.
                // Remember this for the Migrators to come:
        
                // Now we have a valid connection to the storage; with a
                // valid connector user and a valid database schema. This
                // setting is required for successive Migrators.
                $this->State->setHasValidDocStorageConnection(true);
                
            } catch (StorageDatabaseSelectionException $sdse) {
                return $this->addError(
                    'You have chosen to use an existing database
                    \''.$this->State->getDocDatabaseName().'\', but it
                    isn\'t there - or is not accessible with the given
                    connector user. Go back and choose to create a new
                    document database.'
                );
            }
        }

        // ----------------------------------------------------------------

        // Check the versions of the chosen storage layers

        // Remove alpha chars from version string and get storage name
        $sVer = preg_replace('#[^0-9.]#', '', $Storage->getStorageVersion());
        $sName = $Storage->getStorageName();

        // Required version dummy
        $sReqVersion = 'unknown';

        // Check name, _not_ scheme, e.g. MySQL, MySQLi, PostgreSQL, SQLite
        switch (strtolower($sName)) {
            case 'mysql':
                $sReqVersion = self::MYSQL_REQUIRED_VERSION;
                break;

            case 'mysqli':
                $sReqVersion = self::MYSQLI_REQUIRED_VERSION;
                break;

            case 'postgresql':
                $sReqVersion = self::POSTGRESQL_REQUIRED_VERSION;
                break;

            case 'sqlite':
                $sReqVersion = self::SQLITE_REQUIRED_VERSION;
                break;

            default:
                return $this->addError('Unknown storage API.');
        }
        
        // Abuse PHPs version_compare() function
        if (version_compare($sVer, $sReqVersion) == -1) {
            return $this->addError(
                COWIKI_FULL_IDENT . ' requires at least '.$sName.' version '
                . $sReqVersion . ' to work properly. Your '.$sName.' version'
                .' is ' . $sVer . '. Please update your '.$sName.' server'
                .' to the latest available stable version.'
            );
        }

        // Check if we have InnoDB (for MySQL only of course).
        if (substr(strtolower($sName), 0, 5) == 'mysql') {
        
            // We warn only if the user has chosen InnoDB. We do not warn
            // if user selected MyISAM.
            if (substr(strtolower($UriInfo->get('scheme')), -6) == 'innodb') {

                $aData = $Storage->fetchRowAndFreeResult(
                    $Storage->query('SHOW VARIABLES LIKE "HAVE_INNODB"')
                );

                if (!isset($aData[1]) || strtolower($aData[1]) != 'yes') {
                    return $this->addError(
                        'The MySQL database you have chosen does not support
                         the InnoDB storage layer facility. Update or
                         reconfigure the MySQL database.'
                    );
                }
            }
        }
        
        // ----------------------------------------------------------------

        // If we have to create a new database connector user, do not care
        // about the existing one.
        if ($this->State->shallCreateNewDocumentDatabase()) {
            return true;
        }

        // ----------------------------------------------------------------

        // Check the user privileges/permissions
        switch (strtolower($sName)) {

            case 'mysql':     // Make it work with deprecated settings
            case 'mysqli':    // Make it work with deprecated settings

            case 'mysql+innodb':
            case 'mysqli+innodb':
                return $this->checkMySQLUserPrivileges($Storage);
                break;

            case 'postgresql':
                return $this->checkPostgreSQLUserPrivileges($Storage);
                break;

            case 'sqlite':
                return $this->checkSQLiteUserPrivileges($Storage);
                break;
                
            default:
                return $this->addError('Unknown storage API: '.$sName);
        }
        
        return true;
    }

    // --------------------------------------------------------------------

    private function checkMySQLUserPrivileges($Storage) {

        $sHost = $Storage->getHostString();

        // Work around the host string that might contain the port number.
        // Remove the port form that string.
        $sHost = preg_replace('#:[0-9]+#', '', $sHost);
        
        // Determine required connector user privileges. MySQL
        // unfortunately returns only a string that we have to scan for
        // the right parts.
        $sQuery = 'SHOW GRANTS FOR '
                   . $Storage->getUserName()
                   . '@'
                   . $sHost;

        $rResult = $Storage->query($sQuery);

        $sStr = '';
        while ($aData = $Storage->fetchRow($rResult)) {
            $sStr .= $aData[0] . "\n";
        }

        // Check for GRANT ALL PRIVILEGES. This is if user has all
        // privs. This is a bad setting as the _connector_ user has all
        // administrative access to the database, but that is not our
        // problem.
        if (preg_match('#GRANT ALL PRIV#', $sStr)) {
            return;
        }

        // Check for following privileges that are required by the
        // connector user:
        $aRequired = array(
                        'SELECT', 'INSERT', 'UPDATE', 'DELETE',
                        'CREATE', 'DROP', 'INDEX', 'ALTER'
                     );
        $aLack = array();

        foreach ($aRequired as $sVal) {
            if (!preg_match('#'.preg_quote($sVal).'#', $sStr)) {
                $aLack[] = $sVal;
            }
        }

        if (sizeof($aLack)) {
            return $this->addError(
                'The given user \''.$Storage->getUserName().'\' lacks of
                following privileges for the '.$Storage->getStorageName().'
                database \''.$Storage->getDatabase().'\': '
                . join(', ', $aLack) . '.'
            );
        }

    }

    // --------------------------------------------------------------------

    private function checkPostgreSQLUserPrivileges($Storage) {
        return $this->addError(
            'User privilege check not implemented for PostgreSQL. Halt.'
        );
    }

    // --------------------------------------------------------------------

    private function checkSQLiteUserPrivileges($Storage) {
        return $this->addError(
            'User privilege check not implemented for SQLite. Halt.'
        );
    }

} // of class

/*
    Life is a waterfall
    We're one in the river
    And one again after the fall

    Swimming through the void
    We hear the word
    We lose ourselves
    But we find it all ...

    Cause we are the ones that want to play
    Always want to go
    But you never want to stay

    And we are the ones that want to choose
    Always want to play
    But you never want to lose

    Aerials, in the sky
    When you lose small mind
    You free your life

    Life is a waterfall
    We drink from the river
    Then we turn around and put up our walls

    Swimming through the void
    We hear the word
    We lose ourselves
    But we find it all ...

    Cause we are the ones that want to play
    Always want to go
    But you never want to stay

    And we are the ones that want to choose
    Always want to play
    But you never want to lose

    Aerials, in the sky
    When you lose small mind
    You free your life

    Aerials, so up high
    When you free your eyes eternal prize
*/

?>