<?php

/**
 *
 * $Id: class.Migrator_20060320_01.php,v 1.1 2006/03/20 15:53:57 cmarble Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.1 $
 *
 */

/**
 * Migrator implementation
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Archie Campbell, <cephalicmarble@blueyonder.co.uk>
 * @since       coWiki 0.4.0
 */
class Migrator_20060320_01 implements MigratorInterface {

    private
        $State = null;

    // --------------------------------------------------------------------

    public function __construct($StateHolder) {
        $this->State = $StateHolder;
    }

    // --------------------------------------------------------------------

    /**
     *  Get a short description of what this Migrator is going to do
     */
    public function getDescription() {
        return 'Move old stored wiki representation to new, stricter XML';
    }

    // --------------------------------------------------------------------

    /**
     *  Check if this Migrator has to be performed, its task is:
     *
     *  converting old-style documents to newer, stricter xml
     */
    public function isRequired() {

        // Get the resource DSN (database connection details)
        $UriInfo = new UriInfo($this->State->getDocumentStorageDSN());

        // ----------------------------------------------------------------

        $Storage = StorageFactory::getInstance()->create($UriInfo);        
        $Storage->useDatabase($UriInfo->get('basepath'));

        // are there any old-style contents?
        $rResult = $Storage->query(
            "SELECT COUNT(node_id)
               FROM cowiki_node
              WHERE SUBSTRING(content,1,10) != '<document>' ;"
        );

	$aRow = $Storage->fetchRowAndFreeResult($rResult);
        return $aRow[0] > 0;
    }

    // --------------------------------------------------------------------

    /**
     *  Perform the Migrator task
     */
    public function perform() {

        $converter = WikiPseudoXmlConverter :: getInstance();

        // Get the connector user resource DSN (database connection details)
        $UriInfo = new UriInfo($this->State->getDocumentStorageDSN());

        // Connect to the storage/select database
        $Storage = StorageFactory::getInstance()->create($UriInfo);        
        $Storage->useDatabase($UriInfo->get('basepath'));

        // Apply changes
        $Storage->begin();

        // Old wiki xml has no root element
        $rResult = $Storage->query(
                           "SELECT node_id,content
                              FROM cowiki_node
                             WHERE SUBSTRING(content,1,10) != '<document>' ;");

        while($Obj = $Storage->fetchArray($rResult)) {
          //convert old wiki xml to new, strict xml
            $aFields = array('content' => $converter->parse($Obj['content']));
            $aUpdate = array(
                             'table' => 'cowiki_node',
                             'fields' => $aFields,
                             'where' => "node_id = ".$Obj['node_id']
                            );
            $Storage->update($aUpdate);
        }

        $Storage->commit();
    }

} // of class

?>
