<?php

/**
 *
 * $Id: class.StorageMySQLi.php,v 1.19 2005/05/26 23:42:54 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     dao
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.19 $
 *
 */

/**
 * This class is the MySQLi (MyISAM/InnoDB) access abstraction layer.
 * MySQLi stands for 'MySQL improved' and is available since MySQL server
 * version 4.1.
 * It provides basic database access methods like INSERT, UPDATE or just
 * a simple query (SELECT). It is intentionally kept very simple and is
 * only a helper, but not a real persistence layer.
 *
 * @package     dao
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.3.0
 *
 * @todo        [D11N]  Complete documentation
 */
class StorageMySQLi extends StorageMySQL {

    protected $STORAGE_CONTAINER_NAME = 'MySQLi';

    // --------------------------------------------------------------------

    /**
     * Class constructor. Overwrite the parent method.
     *
     * @access  public
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @throws  StorageApiException
     */
    public function __construct() {
        if (!function_exists('mysqli_connect')) {
            throw new StorageApiException('MySQLi not available.');
        }
    }

    // --------------------------------------------------------------------

    public function __destruct() {
        if ($this->rLink) {
            @mysqli_close($this->rLink);
        }
    }

    // --------------------------------------------------------------------

    /**
     * Connect. Overwrite the parent method.
     *
     * @access  public
     * @param   string
     * @param   string
     * @param   string
     * @param   string
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     *
     * @throws  StorageConnectionFailureException
     * @throws  StorageException
     */
    public function connect($sHost, $sPort, $sUser, $sPass) {

        // Remember
        $this->sHost = $sHost;
        $this->sPort = $sPort;
        $this->sUser = $sUser;
        $this->sPass = $sPass;

        $sHostPort = $this->getHostString();
                
        if (!$this->rLink = @mysqli_connect($sHostPort, $sUser, $sPass)) {

            // {{{ DEBUG }}}
            Logger::error(
                'MySQLi error '
                . mysqli_connect_errno() . ': ' . mysqli_connect_error()
            );

            throw new StorageConnectionFailureException(
                mysqli_connect_error()
            );
        }

        // Do not auto commit
        $sQuery = 'SET AUTOCOMMIT=0';

        // {{{ DEBUG }}}
        Logger::sql($sQuery);

        $this->query($sQuery);
    }

    // --------------------------------------------------------------------

    /**
     * Use database
     *
     * @access  public
     * @param   string
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     *
     * @throws  StorageDatabaseSelectionException
     */
    public function useDatabase($sDatabase) {

        // Remember
        $this->sDb = $sDatabase;

        if(!@mysqli_select_db($this->rLink, $sDatabase)) {

            // {{{ DEBUG }}}
            Logger::error(
                'MySQLi error '
                .mysqli_errno($this->rLink).': '.mysqli_error($this->rLink)
            );

            throw new StorageDatabaseSelectionException(
                mysqli_error($this->rLink)
            );
        }
    }

    // --------------------------------------------------------------------

    /**
     * Query. Overwrite the parent method.
     *
     * @access  public
     * @param   string
     * @param   object
     * @return  resource
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$mQueryType"
     * @todo    [D11N]  Check return type
     *
     * @throws  StorageQueryException
     */
    public function query($sQuery, $mQueryType = null) {

        if (!$this->rResult = @mysqli_query($this->rLink, $sQuery)) {

            // {{{ DEBUG }}}
            Logger::error(
                'MySQLi error '
                .mysqli_errno($this->rLink).': '.mysqli_error($this->rLink)
            );

            throw new StorageQueryException(mysqli_error($this->rLink));
        }

        return $this->rResult;
    }

    // --------------------------------------------------------------------

    /**
     * Fetch one single result array and free the occupied memory
     * immediately.
     *
     * @access  public
     * @param   integer
     * @return  array
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$rResource"
     * @todo    [D11N]  Check return type
     */
    public function fetchArray($rResource) {
        $mData = @mysqli_fetch_assoc($rResource);
        if ($mData === false) {
            $this->freeResult($rResource);
        }
    
        return $mData;
    }

    // --------------------------------------------------------------------

    /**
     * Fetch one single result row and free the occupied memory
     * immediately.
     *
     * @access  public
     * @param   integer
     * @return  array
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$rResource"
     * @todo    [D11N]  Check return type
     */
    public function fetchRow($rResource) {
        $mData = @mysqli_fetch_row($rResource);
        if ($mData === false) {
            $this->freeResult($rResource);
        }
    
        return $mData;
    }

    // --------------------------------------------------------------------

    /**
     * Get last insert id
     *
     * @access  public
     * @param   string
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    public function getLastInsertId($sTableName) {
        return @mysqli_insert_id($this->rLink);
    }
    
    // --------------------------------------------------------------------

    /**
     * Free result. Overwrite the parent method.
     *
     * @access  public
     * @param   ressource
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     */
    public function freeResult($rResource) {
        @mysqli_free_result($rResource);
    }

    // --------------------------------------------------------------------

    /**
     * Close current connection
     *
     * @access  public
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     */
    public function close() {
        if ($this->rLink) {
            @mysqli_close($this->rLink);

            $this->rLink = null;
        }
    }

} // of class

/*
    Broken buildings watching over flooded streets where nothing moves
    Shelters for the vultures hardly keeping daylight out
    Marble stained with rust, faded paintings on the walls
    Beaten towers crumble, haunted only by the wind
    
    I will live forever
    No angels left to hold me
    I will be the final man 
*/

?>
