<?php

/**
 *
 * $Id: class.TestWikiParser.php,v 1.4 2006/03/20 15:53:57 cmarble Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     parse
 * @subpackage  class
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.4 $
 *
 */

/**
 * coWiki - coWiki Test parser class
 *
 * @package     parse
 * @subpackage  class
 * @access      public
 *
 * @author      Archie Campbell <cephalicmarble@blueyonder.co.uk>
 * @since       coWiki 0.4.0
 *
 * @todo        [D11N]  Complete documentation
 */
class TestWikiParser extends WikiParser {
    /**
     * Get instance
     *
     * @access  public
     * @return  mixed
     *
     * @author  Archie Campbell <cephalicmarble@blueyonder.co.uk>
     * @since   coWiki 0.4.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    public function getInstance() {
        if (!self::$Instance) {
            self::$Instance = new TestWikiParser;
        }
        return self::$Instance;
    }

    protected function __construct() {}

    // --- Element creators -----------------------------------------------

    /**
     * Create a <link>-element. This method overwrites the parent one.
     *
     * @access  protected
     * @param   string
     * @return  mixed
     *
     * @author  Archie Campbell <cephalicmarble@blueyonder.co.uk>
     * @since   coWiki 0.4.0
     */
    protected function createLinkElement($sStr, $sSquare) {

        $aStr = explode(($sSquare!="square"?")(":"]["),$sStr);

        $sRef   = false;
        $sAlias = false;

        if (isset($aStr[0])) {
            $sRef = trim($aStr[0]);
        }
        if (isset($aStr[1])) {
            $sAlias = trim($aStr[1]);
        }

        // Check if reference string is an URI, if so return the <link>
        // element
        $sPattern = '^(http://|https://|ftp://|mailto:|news:).*';

        if (preg_match('=' . $sPattern . '=', $sRef)) {
            return $this->createUriLinkElement($sRef, $sAlias, $sSquare);
        }

        // Obviously the reference string is not an URI. Check if it has
        // a reference to an other web. Means: "webname|documentname"
        $aWebRef = explode('|', $sRef);

        // Do we have a reference to an other web?
        if (sizeof($aWebRef) > 1) {
            $sWeb = trim($aWebRef[0]);
            $sRef = trim($aWebRef[1]);
            return $this->createGlobalLinkElement($sWeb, $sRef, $sAlias, $sSquare);
        }

        // The reference string was not an URI and it has no reference
        // to an other web -> Look up in database.
        return $this->createLocalLinkElement($sRef, $sAlias, $sSquare);
    }

    /**
     * If a link is an URI
     *
     * @access  protected
     * @param   string
     * @param   string
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     * @author  Archie Campbell <cephalicmarble@blueyonder.co.uk>
     * @since   coWiki 0.4.0
     */
    protected function createUriLinkElement($sRef, $sAlias, $sSquare) {
        $sStr =  "<link><origin><![CDATA[".$sSquare."]]></origin>".
                      "<href><![CDATA[".$sRef."]]></href>";

        if ($sAlias && $sAlias != '') {
            $sStr .= "<![CDATA[".$sAlias."]]>" ;
        }

        $sStr .= "</link>";

        return $sStr;
    }

    /**
     * If a link references to an other web (global)
     *
     * @access  protected
     * @param   string
     * @param   string
     * @param   string
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     * @author  Archie Campbell <cephalicmarble@blueyonder.co.uk>
     * @since   coWiki 0.4.0
     */
    protected function createGlobalLinkElement($sWeb, $sRef, $sAlias, $sSquare) {
        $sWebStr = unescape($sWeb);
        $sRefStr = unescape($sRef);

        /*
        // Create element as though found in database
        $sStr = '<link origin='.($bSquare?'"square"':'"round"').
                        ' idref="' . $Node->get('id') . '">';
        $sStr .=    $sAlias;
        $sStr .= '</link>';
        */

        // Create element as though not found in database
        $sStr = "<link><origin><![CDATA[".$sSquare."]]></origin>".
                         "<strref><![CDATA[".$sWeb.''.$sRef."]]></strref>";
        $sStr .= "<![CDATA[".$sAlias."]]>";
        $sStr .= "</link>";

        return $sStr;
    }

    /**
     * If a link references within a web only (local)
     *
     * @access  protected
     * @param   string
     * @param   string
     * @return  string
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     * @author  Archie Campbell <cephalicmarble@blueyonder.co.uk>
     * @since   coWiki 0.4.0
     */
    protected function createLocalLinkElement($sRef, $sAlias, $sSquare) {
        $aData   = false;
        $sRefStr = unescape($sRef);

        /*
            // Create element as though found in database
            $sStr = '<link origin='.($bSquare?'"square"':'"round"').
                            ' idref="' . $NewNode->get('id') . '">';
            $sStr .=    $sAlias;
            $sStr .= '</link>';
        */

        // Create element as though not found in database
        $sStr = "<link><origin><![CDATA[".$sSquare."]]></origin>".
                         "<strref><![CDATA[".$sRef."]]></strref>";
        $sStr .= "<![CDATA[".$sAlias."]]>";
        $sStr .= "</link>";

        return $sStr;
    }

} // of class

?>
