<?php

/**
 *
 * $Id: class.PrivateFrontMenuDefaultDisplay.php,v 1.14 2005/09/12 22:48:00 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * <pre>
 * #name:      PrivateFrontMenuDefaultDisplay
 * #purpose:   Provide the default (main) menu
 * #param:     none
 * #caching:   yes, internal cache
 * #comment:   none
 * #version:   1.0
 * #date:      01. November 2002
 * #author:    Daniel T. Gorski <daniel.gorski@develnet.org>
 *
 * Please read and understand the README.PLUGIN file before you touch
 * something here.
 * </pre>
 *
 * @package     plugin
 * @subpackage  PrivateFront
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.14 $
 *
 */

/**
 * coWiki - Provide the default (main) menu
 *
 * @package     plugin
 * @subpackage  PrivateFront
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.3.0
 *
 * @todo        [D11N]  Complete documentation
 */
class PrivateFrontMenuDefaultDisplay extends AbstractPlugin
                                     implements CustomObserver {

    // Put in the interface version the plugin works with
    const REQUIRED_INTERFACE_VERSION = 1;

    // --------------------------------------------------------------------

    /**
     * Init
     *
     * @access  public
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check return type
     */
    public function init() {
        return parent::init(self::REQUIRED_INTERFACE_VERSION);
    }

    // --------------------------------------------------------------------

    /**
     * Perform
     *
     * @access  public
     * @return  boolean
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [FIX]   Relocate the HTML to template. Maybe a TemplateProcessor
     */
    public function perform() {

        // Make this plugin observe the document DAO and user DAO.
        // Observed DAO will signal if this plugin should clean up its
        // cache. See "update()"
        $this->Context->getDocumentDAO()->addObserver($this);
        $this->Context->getUserDAO()->addObserver($this);

        // ----------------------------------------------------------------

        // If cached result exists, put it out and leave the plugin
        if ($sStr = $this->Context->getFromCache($this)) {
            echo $sStr;
            return true; // leave plugin
        }

        // ----------------------------------------------------------------

        // Get plugin parameters
        $sImg = $this->Context->hasPluginParam('img')
                  ? $this->Context->getPluginParam('img')
                  : false;

        $sHspace = $this->Context->hasPluginParam('imghspace')
                    ? $this->Context->getPluginParam('imghspace')
                    : '0';

        $sVspace = $this->Context->hasPluginParam('imgvspace')
                    ? $this->Context->getPluginParam('imgvspace')
                    : '0';

        // ----------------------------------------------------------------

        // Get image dimensions
        $sDocRoot = $this->Context->getEnvironment()->get('DOCUMENT_ROOT');
        $sFile = $sDocRoot . $this->Registry->get('PATH_IMAGES') . $sImg;

        $sWidth  = '0';
        $sHeight = '0';

        if ($sImg) {
            if (is_readable($sFile)) {
                $aProp = @getimagesize($sFile);
                $sWidth = $aProp[0];
                $sHeight = $aProp[1];
            }
        }

        $this->Template->set('TPL_BULLET_IMAGE',  $sImg);
        $this->Template->set('TPL_BULLET_HSPACE', $sHspace);
        $this->Template->set('TPL_BULLET_VSPACE', $sVspace);
        if ($sHeight != '0' && $sWidth != '0') {
            $this->Template->set('TPL_BULLET_WIDTH',  $sWidth);
            $this->Template->set('TPL_BULLET_HEIGHT', $sHeight);
        }

        // ----------------------------------------------------------------

        // Get variables from template
        $Tpl = $this->Context->getTemplateProcessor();
        $aVars = $Tpl->fetchVariables('plugin.front.menu.default.tpl');

        // Store variable, make it accessible in helper function
        $this->sBullet = @$aVars['TPL_BULLET'];

        // ----------------------------------------------------------------

        // Get webs as composite
        $Node = $this->Context->getDocumentDAO()->getWebComposite();

        $bSeparate = false;
        $aTplItem = array();

        // Iterate through webs
        $It = $Node->getItems()->iterator();

        while ($Obj = $It->next()) {

            if (!$Obj->get('isInMenu') || !$Obj->isReadable()) {
                continue;
            }

            $aItem = array();

            $aItem['INDENT'] = '';
            $aItem['IMAGE']  = $this->sBullet;
            $aItem['NAME']   = escape($Obj->get('name'));
            $aItem['HREF']   = $this->Response->getControllerHref(
                                  'node='.$Obj->get('id')
                               );

            // Odd separator
            if ($bSeparate) {
                $aItem['SEPARATOR'] = @$aVars['TPL_SEPARATOR'];
            }
            $bSeparate = true;

            // Append item to template items
            $aTplItem[] = $aItem;

            // If item has children, append them to menu items
            $aChildMenu = $this->getMenuItems(
                            $Obj,
                            $this->Registry->get('MENU_MAX_DEPTH')
                          );

            for ($j=0, $m=sizeof($aChildMenu); $j<$m; $j++) {
                $aTplItem[] = $aChildMenu[$j];
            }
        }

        $this->Template->set('TPL_ITEM', $aTplItem);

        // ----------------------------------------------------------------

        // Parse template
        $sStr = $Tpl->parse('plugin.front.menu.default.tpl');

        // Cache result
        $this->Context->putToCache($this, $sStr);

        // Output result
        echo $sStr;
    }

    // --------------------------------------------------------------------

    /**
     * Get menu items
     *
     * @access  private
     * @param   object
     * @param   integer
     * @return  mixed
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     *
     * @todo    [D11N]  Check description
     * @todo    [D11N]  Check the parameter type of "$Node"
     * @todo    [D11N]  Check the parameter type of "$nDepth"
     */
    private function getMenuItems($Node, $nDepth) {
        static $nLevel = 1;

        $aTplItem = array();

        // Maximum depth
        if ($nLevel >= $nDepth) {
            return $aTplItem;
        }

        // Get all container children
        $Node = $this->Context->getDocumentDAO()->getAllChildren($Node);

        // ---

        // Iterate through children
        $It = $Node->getItems()->iterator();

        while ($Obj = $It->next()) {
            if (!$Obj->get('isInMenu') || !$Obj->isReadable()) {
                continue;
            }

            $aItem = array();

            $aItem['INDENT'] = str_repeat('&nbsp;&nbsp;', $nLevel);
            $aItem['IMAGE']  = $this->sBullet;
            $aItem['NAME']   = escape($Obj->get('name'));
            $aItem['HREF']   = $this->Response->getControllerHref(
                                  'node='.$Obj->get('id')
                               );

            $aTplItem[] = $aItem;

            // If item has children, append them to menu items
            $nLevel++;
            $aChildMenu = $this->getMenuItems($Obj, $nDepth);
            $nLevel--;

            for ($j=0, $m=sizeof($aChildMenu); $j<$m; $j++) {
                $aTplItem[] = $aChildMenu[$j];
            }
        }

        return $aTplItem;
    }

    // --------------------------------------------------------------------

    /**
     * Implement observer "update()" method to clean the cache if necessary
     *
     * @access  public
     * @return  void
     *
     * @author  Daniel T. Gorski, <daniel.gorski@develnet.org>
     * @since   coWiki 0.3.0
     */
    public function update() {
        $this->Context->removeFromCache($this);
    }

} // of plugin component

?>